@@ -51,30 +51,36 @@ test ${kernel:0:5} = "MINGW" && kernel="MINGW"
 
 compiler="gcc"
 
-qt_version=5.5.1
+qt_version=5.6.1
 zlib_version=1.2.8
 zlib_patchlevel=2
 libogg_version=1.3.2
 libogg_patchlevel=1
-libvorbis_version=1.3.4
+libvorbis_version=1.3.5
 libvorbis_patchlevel=3
-ffmpeg_version=2.8.2
-ffmpeg_patchlevel=1
+ffmpeg_version=3.0.1
+ffmpeg_patchlevel=3
 #libav_version=11.4
 #libav_patchlevel=2
 libflac_version=1.3.1
 libflac_patchlevel=4
 id3lib_version=3.8.3
 id3lib_patchlevel=16.2
-taglib_version=1.10
-chromaprint_version=1.2
+taglib_version=1.11
+chromaprint_version=1.3
 chromaprint_patchlevel=1
 mp4v2_version=2.0.0
 mp4v2_patchlevel=4
 
+FLAC_BUILD_OPTION="--enable-debug"
+ID3LIB_BUILD_OPTION="--enable-debug=minimum"
+AV_BUILD_OPTION="--enable-debug=1"
+CMAKE_BUILD_OPTION="-DCMAKE_BUILD_TYPE=RelWithDebInfo"
 # Uncomment for debug build
-#ENABLE_DEBUG=--enable-debug
-#CMAKE_BUILD_TYPE_DEBUG="-DCMAKE_BUILD_TYPE=Debug"
+#FLAC_BUILD_OPTION="--enable-debug"
+#ID3LIB_BUILD_OPTION="--enable-debug=yes"
+#AV_BUILD_OPTION="--enable-debug=3"
+#CMAKE_BUILD_OPTION="-DCMAKE_BUILD_TYPE=Debug"
 
 if ! which cmake >/dev/null; then
   echo cmake not found.
@@ -196,8 +202,8 @@ test -f zlib_${zlib_version}.dfsg.orig.tar.gz ||
 $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/z/zlib/zlib_${zlib_version}.dfsg.orig.tar.gz
 
 if test -n "${ffmpeg_version}"; then
-test -f ffmpeg_${ffmpeg_version}.orig.tar.gz ||
-$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/f/ffmpeg/ffmpeg_${ffmpeg_version}.orig.tar.gz
+test -f ffmpeg_${ffmpeg_version}.orig.tar.xz ||
+$DOWNLOAD http://ftp.de.debian.org/debian/pool/main/f/ffmpeg/ffmpeg_${ffmpeg_version}.orig.tar.xz
 test -f ffmpeg_${ffmpeg_version}-${ffmpeg_patchlevel}.debian.tar.xz ||
 $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/f/ffmpeg/ffmpeg_${ffmpeg_version}-${ffmpeg_patchlevel}.debian.tar.xz
 ffmpeg_dir=ffmpeg-${ffmpeg_version}
@@ -231,7 +237,7 @@ $DOWNLOAD http://ftp.de.debian.org/debian/pool/main/m/mp4v2/mp4v2_${mp4v2_versio
 if test "$compiler" = "cross-mingw"; then
 test -f mingw.cmake ||
 cat >mingw.cmake <<EOF
-set(QT_PREFIX /windows/Qt/Qt${qt_version}/${qt_version%.?}/mingw492_32)
+set(QT_PREFIX /windows/Qt/Qt${qt_version}/${qt_version%.?}/mingw49_32)
 
 set(CMAKE_SYSTEM_NAME Windows)
 set(CMAKE_C_COMPILER ${cross_host}-gcc)
@@ -402,6 +408,22 @@ diff -ru id3lib-3.8.3.orig/include/id3/globals.h id3lib-3.8.3/include/id3/global
  #  define LINKOPTION_USE_DYNAMIC    3 //if your project links id3lib dynamic
 EOF
 
+test -f id3lib-3.8.3_wintempfile.patch ||
+cat >id3lib-3.8.3_wintempfile.patch <<"EOF"
+diff -ru id3lib-3.8.3.orig/src/tag_file.cpp id3lib-3.8.3/src/tag_file.cpp
+--- id3lib-3.8.3.orig/src/tag_file.cpp	2016-06-09 20:54:44.395068889 +0200
++++ id3lib-3.8.3/src/tag_file.cpp	2016-06-09 21:39:35.044411098 +0200
+@@ -242,7 +242,7 @@
+     strcpy(sTempFile, filename.c_str());
+     strcat(sTempFile, sTmpSuffix.c_str());
+ 
+-#if !defined(HAVE_MKSTEMP)
++#if !defined(HAVE_MKSTEMP) || defined WIN32
+     // This section is for Windows folk
+     fstream tmpOut;
+     createFile(sTempFile, tmpOut);
+EOF
+
 test -f taglib-msvc.patch ||
 cat >taglib-msvc.patch <<"EOF"
 diff -ru taglib-1.8.orig/CMakeLists.txt taglib-1.8/CMakeLists.txt
@@ -524,287 +546,6 @@ cat >taglib_bvreplace.patch <<"EOF"
  }
 EOF
 
-test -f taglib_podcast.patch ||
-cat >taglib_podcast.patch <<"EOF"
-diff --git a/taglib/CMakeLists.txt b/taglib/CMakeLists.txt
-index 73c1a2f..31e2c49 100644
---- a/taglib/CMakeLists.txt
-+++ b/taglib/CMakeLists.txt
-@@ -83,6 +83,7 @@ set(tag_HDRS
-   mpeg/id3v2/frames/urllinkframe.h
-   mpeg/id3v2/frames/chapterframe.h
-   mpeg/id3v2/frames/tableofcontentsframe.h
-+  mpeg/id3v2/frames/podcastframe.h
-   ogg/oggfile.h
-   ogg/oggpage.h
-   ogg/oggpageheader.h
-@@ -177,6 +178,7 @@ set(frames_SRCS
-   mpeg/id3v2/frames/urllinkframe.cpp
-   mpeg/id3v2/frames/chapterframe.cpp
-   mpeg/id3v2/frames/tableofcontentsframe.cpp
-+  mpeg/id3v2/frames/podcastframe.cpp
- )
- 
- set(ogg_SRCS
-diff --git a/taglib/mpeg/id3v2/frames/podcastframe.cpp b/taglib/mpeg/id3v2/frames/podcastframe.cpp
-new file mode 100644
-index 0000000..5115a7d
---- /dev/null
-+++ b/taglib/mpeg/id3v2/frames/podcastframe.cpp
-@@ -0,0 +1,79 @@
-+/***************************************************************************
-+    copyright            : (C) 2015 by Urs Fleisch
-+    email                : ufleisch@users.sourceforge.net
-+ ***************************************************************************/
-+
-+/***************************************************************************
-+ *   This library is free software; you can redistribute it and/or modify  *
-+ *   it  under the terms of the GNU Lesser General Public License version  *
-+ *   2.1 as published by the Free Software Foundation.                     *
-+ *                                                                         *
-+ *   This library is distributed in the hope that it will be useful, but   *
-+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
-+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
-+ *   Lesser General Public License for more details.                       *
-+ *                                                                         *
-+ *   You should have received a copy of the GNU Lesser General Public      *
-+ *   License along with this library; if not, write to the Free Software   *
-+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
-+ *   02110-1301  USA                                                       *
-+ *                                                                         *
-+ *   Alternatively, this file is available under the Mozilla Public        *
-+ *   License Version 1.1.  You may obtain a copy of the License at         *
-+ *   http://www.mozilla.org/MPL/                                           *
-+ ***************************************************************************/
-+
-+#include "podcastframe.h"
-+
-+using namespace TagLib;
-+using namespace ID3v2;
-+
-+class PodcastFrame::PodcastFramePrivate
-+{
-+public:
-+  ByteVector fieldData;
-+};
-+
-+////////////////////////////////////////////////////////////////////////////////
-+// public members
-+////////////////////////////////////////////////////////////////////////////////
-+
-+PodcastFrame::PodcastFrame() : Frame("PCST")
-+{
-+  d = new PodcastFramePrivate;
-+  d->fieldData = ByteVector(4, '\0');
-+}
-+
-+PodcastFrame::~PodcastFrame()
-+{
-+  delete d;
-+}
-+
-+String PodcastFrame::toString() const
-+{
-+  return String();
-+}
-+
-+////////////////////////////////////////////////////////////////////////////////
-+// protected members
-+////////////////////////////////////////////////////////////////////////////////
-+
-+void PodcastFrame::parseFields(const ByteVector &data)
-+{
-+  d->fieldData = data;
-+}
-+
-+ByteVector PodcastFrame::renderFields() const
-+{
-+  return d->fieldData;
-+}
-+
-+////////////////////////////////////////////////////////////////////////////////
-+// private members
-+////////////////////////////////////////////////////////////////////////////////
-+
-+PodcastFrame::PodcastFrame(const ByteVector &data, Header *h) : Frame(h)
-+{
-+  d = new PodcastFramePrivate;
-+  parseFields(fieldData(data));
-+}
-diff --git a/taglib/mpeg/id3v2/frames/podcastframe.h b/taglib/mpeg/id3v2/frames/podcastframe.h
-new file mode 100644
-index 0000000..7bbc213
---- /dev/null
-+++ b/taglib/mpeg/id3v2/frames/podcastframe.h
-@@ -0,0 +1,80 @@
-+/***************************************************************************
-+    copyright            : (C) 2015 by Urs Fleisch
-+    email                : ufleisch@users.sourceforge.net
-+ ***************************************************************************/
-+
-+/***************************************************************************
-+ *   This library is free software; you can redistribute it and/or modify  *
-+ *   it  under the terms of the GNU Lesser General Public License version  *
-+ *   2.1 as published by the Free Software Foundation.                     *
-+ *                                                                         *
-+ *   This library is distributed in the hope that it will be useful, but   *
-+ *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
-+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
-+ *   Lesser General Public License for more details.                       *
-+ *                                                                         *
-+ *   You should have received a copy of the GNU Lesser General Public      *
-+ *   License along with this library; if not, write to the Free Software   *
-+ *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA         *
-+ *   02110-1301  USA                                                       *
-+ *                                                                         *
-+ *   Alternatively, this file is available under the Mozilla Public        *
-+ *   License Version 1.1.  You may obtain a copy of the License at         *
-+ *   http://www.mozilla.org/MPL/                                           *
-+ ***************************************************************************/
-+
-+#ifndef TAGLIB_PODCASTFRAME_H
-+#define TAGLIB_PODCASTFRAME_H
-+
-+#include "id3v2frame.h"
-+#include "taglib_export.h"
-+
-+namespace TagLib {
-+
-+  namespace ID3v2 {
-+
-+    //! ID3v2 podcast frame
-+    /*!
-+     * An implementation of ID3v2 podcast flag, a frame with four zero bytes.
-+     */
-+    class TAGLIB_EXPORT PodcastFrame : public Frame
-+    {
-+      friend class FrameFactory;
-+
-+    public:
-+      /*!
-+       * Construct a podcast frame.
-+       */
-+      PodcastFrame();
-+
-+      /*!
-+       * Destroys this PodcastFrame instance.
-+       */
-+      virtual ~PodcastFrame();
-+
-+      /*!
-+       * Returns a null string.
-+       */
-+      virtual String toString() const;
-+
-+    protected:
-+      // Reimplementations.
-+
-+      virtual void parseFields(const ByteVector &data);
-+      virtual ByteVector renderFields() const;
-+
-+    private:
-+      /*!
-+       * The constructor used by the FrameFactory.
-+       */
-+      PodcastFrame(const ByteVector &data, Header *h);
-+      PodcastFrame(const PodcastFrame &);
-+      PodcastFrame &operator=(const PodcastFrame &);
-+
-+      class PodcastFramePrivate;
-+      PodcastFramePrivate *d;
-+    };
-+
-+  }
-+}
-+#endif
-diff --git a/taglib/mpeg/id3v2/id3v2frame.cpp b/taglib/mpeg/id3v2/id3v2frame.cpp
-index bee5375..4313bcc 100644
---- a/taglib/mpeg/id3v2/id3v2frame.cpp
-+++ b/taglib/mpeg/id3v2/id3v2frame.cpp
-@@ -117,7 +117,8 @@ Frame *Frame::createTextualFrame(const String &key, const StringList &values) //
-   // check if the key is contained in the key<=>frameID mapping
-   ByteVector frameID = keyToFrameID(key);
-   if(!frameID.isNull()) {
--    if(frameID[0] == 'T'){ // text frame
-+    // Apple proprietary WFED (Podcast URL) is in fact a text frame.
-+    if(frameID[0] == 'T' || frameID == "WFED"){ // text frame
-       TextIdentificationFrame *frame = new TextIdentificationFrame(frameID, String::UTF8);
-       frame->setText(values);
-       return frame;
-@@ -427,6 +428,12 @@ static const char *frameTranslation[][2] = {
-   // Other frames
-   { "COMM", "COMMENT" },
-   //{ "USLT", "LYRICS" }, handled specially
-+  // Apple iTunes proprietary frames
-+  { "PCST", "PODCAST" },
-+  { "TCAT", "PODCASTCATEGORY" },
-+  { "TDES", "PODCASTDESC" },
-+  { "TGID", "PODCASTID" },
-+  { "WFED", "PODCASTURL" },
- };
- 
- static const TagLib::uint txxxFrameTranslationSize = 8;
-@@ -532,7 +539,8 @@ PropertyMap Frame::asProperties() const
-   // workaround until this function is virtual
-   if(id == "TXXX")
-     return dynamic_cast< const UserTextIdentificationFrame* >(this)->asProperties();
--  else if(id[0] == 'T')
-+  // Apple proprietary WFED (Podcast URL) is in fact a text frame.
-+  else if(id[0] == 'T' || id == "WFED")
-     return dynamic_cast< const TextIdentificationFrame* >(this)->asProperties();
-   else if(id == "WXXX")
-     return dynamic_cast< const UserUrlLinkFrame* >(this)->asProperties();
-diff --git a/taglib/mpeg/id3v2/id3v2framefactory.cpp b/taglib/mpeg/id3v2/id3v2framefactory.cpp
-index f6a4aac..bf4b0ee 100644
---- a/taglib/mpeg/id3v2/id3v2framefactory.cpp
-+++ b/taglib/mpeg/id3v2/id3v2framefactory.cpp
-@@ -49,6 +49,7 @@
- #include "frames/eventtimingcodesframe.h"
- #include "frames/chapterframe.h"
- #include "frames/tableofcontentsframe.h"
-+#include "frames/podcastframe.h"
- 
- using namespace TagLib;
- using namespace ID3v2;
-@@ -167,7 +168,8 @@ Frame *FrameFactory::createFrame(const ByteVector &origData, Header *tagHeader)
- 
-   // Text Identification (frames 4.2)
- 
--  if(frameID.startsWith("T")) {
-+  // Apple proprietary WFED (Podcast URL) is in fact a text frame.
-+  if(frameID.startsWith("T") || frameID == "WFED") {
- 
-     TextIdentificationFrame *f = frameID != "TXXX"
-       ? new TextIdentificationFrame(data, header)
-@@ -287,6 +289,11 @@ Frame *FrameFactory::createFrame(const ByteVector &origData, Header *tagHeader)
-   if(frameID == "CTOC")
-     return new TableOfContentsFrame(tagHeader, data, header);
- 
-+  // Apple proprietary PCST (Podcast)
-+
-+  if(frameID == "PCST")
-+    return new PodcastFrame(data, header);
-+
-   return new UnknownFrame(data, header);
- }
- 
-@@ -423,6 +430,14 @@ bool FrameFactory::updateFrame(Frame::Header *header) const
-     convertFrame("WPB", "WPUB", header);
-     convertFrame("WXX", "WXXX", header);
- 
-+    // Apple iTunes nonstandard frames
-+    convertFrame("PCS", "PCST", header);
-+    convertFrame("TCT", "TCAT", header);
-+    convertFrame("TDR", "TDRL", header);
-+    convertFrame("TDS", "TDES", header);
-+    convertFrame("TID", "TGID", header);
-+    convertFrame("WFD", "WFED", header);
-+
-     break;
-   }
- 
-EOF
-
 test -f mp4v2_win32.patch ||
 cat >mp4v2_win32.patch <<"EOF"
 diff -ruN mp4v2-2.0.0.orig/GNUmakefile.am mp4v2-2.0.0/GNUmakefile.am
@@ -2000,6 +1741,87 @@ diff -ruN mp4v2-2.0.0.orig/libplatform/platform_win32_impl.h mp4v2-2.0.0/libplat
 +
 +}}} // namespace mp4v2::platform::win32
 EOF
+
+test -f vorbis_alloc_on_heap.patch ||
+cat >vorbis_alloc_on_heap.patch <<"EOF"
+From: Ralph Giles <giles@thaumas.net>
+Date: Tue, 13 Oct 2015 22:32:59 +0000 (-0700)
+Subject: Allocate comment temporaries on the heap.
+X-Git-Url: https://git.xiph.org/?p=vorbis.git;a=commitdiff_plain;h=c75b3b1282de1010883aa1391bc8ea31dc8ac98e
+
+Allocate comment temporaries on the heap.
+
+Use malloc/free instead of the more convenient alloca for
+comment data. Album art can easily be larger than the local
+stack limit and crash the process.
+
+Thanks to Robert Kausch for the suggestion.
+---
+
+diff --git a/lib/info.c b/lib/info.c
+index e447a0c..b8f25ee 100644
+--- a/lib/info.c
++++ b/lib/info.c
+@@ -65,11 +65,13 @@ void vorbis_comment_add(vorbis_comment *vc,const char *comment){
+ }
+ 
+ void vorbis_comment_add_tag(vorbis_comment *vc, const char *tag, const char *contents){
+-  char *comment=alloca(strlen(tag)+strlen(contents)+2); /* +2 for = and \0 */
++  /* Length for key and value +2 for = and \0 */
++  char *comment=_ogg_malloc(strlen(tag)+strlen(contents)+2);
+   strcpy(comment, tag);
+   strcat(comment, "=");
+   strcat(comment, contents);
+   vorbis_comment_add(vc, comment);
++  _ogg_free(comment);
+ }
+ 
+ /* This is more or less the same as strncasecmp - but that doesn't exist
+@@ -88,27 +90,30 @@ char *vorbis_comment_query(vorbis_comment *vc, const char *tag, int count){
+   long i;
+   int found = 0;
+   int taglen = strlen(tag)+1; /* +1 for the = we append */
+-  char *fulltag = alloca(taglen+ 1);
++  char *fulltag = _ogg_malloc(taglen+1);
+ 
+   strcpy(fulltag, tag);
+   strcat(fulltag, "=");
+ 
+   for(i=0;i<vc->comments;i++){
+     if(!tagcompare(vc->user_comments[i], fulltag, taglen)){
+-      if(count == found)
++      if(count == found) {
+         /* We return a pointer to the data, not a copy */
+-              return vc->user_comments[i] + taglen;
+-      else
++        _ogg_free(fulltag);
++        return vc->user_comments[i] + taglen;
++      } else {
+         found++;
++      }
+     }
+   }
++  _ogg_free(fulltag);
+   return NULL; /* didn't find anything */
+ }
+ 
+ int vorbis_comment_query_count(vorbis_comment *vc, const char *tag){
+   int i,count=0;
+   int taglen = strlen(tag)+1; /* +1 for the = we append */
+-  char *fulltag = alloca(taglen+1);
++  char *fulltag = _ogg_malloc(taglen+1);
+   strcpy(fulltag,tag);
+   strcat(fulltag, "=");
+ 
+@@ -117,6 +122,7 @@ int vorbis_comment_query_count(vorbis_comment *vc, const char *tag){
+       count++;
+   }
+ 
++  _ogg_free(fulltag);
+   return count;
+ }
+ 
+EOF
 cd ..
 
 
@@ -2033,6 +1855,9 @@ tar xzf source/libvorbis_${libvorbis_version}.orig.tar.gz
 cd libvorbis-${libvorbis_version}/
 unxz -c ../source/libvorbis_${libvorbis_version}-${libvorbis_patchlevel}.debian.tar.xz | tar x
 for f in $(cat debian/patches/series); do patch -p1 <debian/patches/$f; done
+if test $libvorbis_version = "1.3.5"; then
+  patch -p1 <../source/vorbis_alloc_on_heap.patch
+fi
 test -f win32/VS2010/libogg.props.orig || mv win32/VS2010/libogg.props win32/VS2010/libogg.props.orig
 sed "s/<LIBOGG_VERSION>1.2.0</<LIBOGG_VERSION>$libogg_version</" win32/VS2010/libogg.props.orig >win32/VS2010/libogg.props
 cd ..
@@ -2061,6 +1886,7 @@ cd id3lib-${id3lib_version}/
 unxz -c ../source/id3lib3.8.3_${id3lib_version}-${id3lib_patchlevel}.debian.tar.xz | tar x
 for f in $(cat debian/patches/series); do patch --binary -p1 <debian/patches/$f; done
 patch -p1 <../source/id3lib-3.8.3_mingw.patch
+patch -p1 <../source/id3lib-3.8.3_wintempfile.patch
 test -f makefile.win32.orig || mv makefile.win32 makefile.win32.orig
 sed 's/-W3 -WX -GX/-W3 -EHsc/; s/-MD -D "WIN32" -D "_DEBUG"/-MDd -D "WIN32" -D "_DEBUG"/' makefile.win32.orig >makefile.win32
 cd ..
@@ -2084,10 +1910,13 @@ else
   sed -i 's/^ADD_SUBDIRECTORY(bindings)/#ADD_SUBDIRECTORY(bindings)/' ./CMakeLists.txt
   sed -i 's/^ADD_LIBRARY(tag SHARED/ADD_LIBRARY(tag STATIC/' ./taglib/CMakeLists.txt
 fi
+if test $taglib_nr -ge 111; then
+  taglib_static_option=-DBUILD_SHARED_LIBS=OFF
+else
+  taglib_static_option=-DENABLE_STATIC=ON
+fi
 if test "${taglib_version}" = "1.9.1"; then
   patch -p1 <../source/taglib_bvreplace.patch
-elif test "${taglib_version:0:4}" = "1.10"; then
-  patch -p1 <../source/taglib_podcast.patch
 fi
 cd ..
 fi
@@ -2095,7 +1924,7 @@ fi
 echo "### Extracting ffmpeg"
 
 if test -n "${ffmpeg_version}"; then
-tar xzf source/ffmpeg_${ffmpeg_version}.orig.tar.gz 
+unxz -c source/ffmpeg_${ffmpeg_version}.orig.tar.xz | tar x || true
 cd ffmpeg-${ffmpeg_version}/
 unxz -c ../source/ffmpeg_${ffmpeg_version}-${ffmpeg_patchlevel}.debian.tar.xz | tar x
 for f in $(cat debian/patches/series); do patch -p1 <debian/patches/$f; done
@@ -2181,7 +2010,7 @@ test -d buildroot || mkdir buildroot
 _buildroot=$(pwd)/buildroot
 
 cd taglib-${taglib_version}/
-cmake -DWITH_ASF=ON -DWITH_MP4=ON -DENABLE_STATIC=ON -DCMAKE_BUILD_TYPE=Release -DCMAKE_INSTALL_PREFIX=$_buildroot/usr/local -DANDROID_NDK=$_android_ndk_root -DANDROID_ABI=$_android_abi -DANDROID_TOOLCHAIN_PREFIX=$_android_toolchain_prefix -DCMAKE_TOOLCHAIN_FILE=../../kid3/android/qt-android-cmake/toolchain/android.toolchain.cmake
+cmake -DWITH_ASF=ON -DWITH_MP4=ON $taglib_static_option -DCMAKE_BUILD_TYPE=Release -DCMAKE_INSTALL_PREFIX=$_buildroot/usr/local -DANDROID_NDK=$_android_ndk_root -DANDROID_ABI=$_android_abi -DANDROID_TOOLCHAIN_PREFIX=$_android_toolchain_prefix -DCMAKE_TOOLCHAIN_FILE=../../kid3/android/qt-android-cmake/toolchain/android.toolchain.cmake
 make install
 cd ..
 
@@ -2195,7 +2024,7 @@ _android_abi=$_android_abi
 _android_toolchain_prefix=$_android_toolchain_prefix
 _android_qt_root=$_android_qt_root
 _buildprefix=\$(cd ..; pwd)/buildroot/usr/local
-cmake -GNinja -DJAVA_HOME=\$_java_root -DQT_ANDROID_SDK_ROOT=\$_android_sdk_root -DANDROID_NDK=\$_android_ndk_root -DQT_ANDROID_ANT=/usr/bin/ant -DAPK_ALL_TARGET=OFF -DANDROID_ABI=\$_android_abi -DANDROID_TOOLCHAIN_PREFIX=\$_android_toolchain_prefix -DCMAKE_TOOLCHAIN_FILE=../../kid3/android/qt-android-cmake/toolchain/android.toolchain.cmake -DQT_QMAKE_EXECUTABLE=\$_android_qt_root/bin/qmake -DCMAKE_BUILD_TYPE=Release -DDOCBOOK_XSL_DIR=/usr/share/xml/docbook/stylesheet/nwalsh -DPERL_EXECUTABLE=/usr/bin/perl -DXSLTPROC=/usr/bin/xsltproc -DGZIP_EXECUTABLE=/bin/gzip -DTAGLIBCONFIG_EXECUTABLE=\$_buildprefix/bin/taglib-config ../../kid3
+cmake -DJAVA_HOME=\$_java_root -DQT_ANDROID_SDK_ROOT=\$_android_sdk_root -DANDROID_NDK=\$_android_ndk_root -DQT_ANDROID_ANT=/usr/bin/ant -DAPK_ALL_TARGET=OFF -DANDROID_ABI=\$_android_abi -DANDROID_TOOLCHAIN_PREFIX=\$_android_toolchain_prefix -DCMAKE_TOOLCHAIN_FILE=../../kid3/android/qt-android-cmake/toolchain/android.toolchain.cmake -DQT_QMAKE_EXECUTABLE=\$_android_qt_root/bin/qmake -DCMAKE_BUILD_TYPE=Release -DDOCBOOK_XSL_DIR=/usr/share/xml/docbook/stylesheet/nwalsh -DPYTHON_EXECUTABLE=/usr/bin/python -DXSLTPROC=/usr/bin/xsltproc -DGZIP_EXECUTABLE=/bin/gzip -DTAGLIBCONFIG_EXECUTABLE=\$_buildprefix/bin/taglib-config ../../kid3
 EOF
   chmod +x kid3/build.sh
 fi
@@ -2245,7 +2074,7 @@ cd ../..
 echo "### Building taglib"
 
 cd taglib-${taglib_version}/
-test -f taglib.sln || cmake -G "Visual Studio 11" -DWITH_ASF=ON -DWITH_MP4=ON -DENABLE_STATIC=ON -DCMAKE_INSTALL_PREFIX=
+test -f taglib.sln || cmake -G "Visual Studio 11" -DWITH_ASF=ON -DWITH_MP4=ON $taglib_static_option -DCMAKE_INSTALL_PREFIX=
 mkdir -p instd
 DESTDIR=instd cmake --build . --config Debug --target install
 mkdir -p inst
@@ -2287,10 +2116,10 @@ if test $kernel = "MINGW"; then
 make -f win32/Makefile.gcc
 make install -f win32/Makefile.gcc INCLUDE_PATH=`pwd`/inst/usr/local/include LIBRARY_PATH=`pwd`/inst/usr/local/lib BINARY_PATH=`pwd`/inst/usr/local/bin
 elif test "$compiler" = "cross-mingw"; then
-make -f win32/Makefile.gcc PREFIX=${cross_host}-
+make -f win32/Makefile.gcc LOC=-g PREFIX=${cross_host}-
 make install -f win32/Makefile.gcc INCLUDE_PATH=`pwd`/inst/usr/local/include LIBRARY_PATH=`pwd`/inst/usr/local/lib BINARY_PATH=`pwd`/inst/usr/local/bin
 else
-CFLAGS="$CFLAGS -O3 -Wall -DNO_FSEEKO" ./configure --static
+CFLAGS="$CFLAGS -g -O3 -Wall -DNO_FSEEKO" ./configure --static
 sed 's/LIBS=$(STATICLIB) $(SHAREDLIB) $(SHAREDLIBV)/LIBS=$(STATICLIB)/' Makefile >Makefile.inst
 mkdir -p inst/usr/local
 make install -f Makefile.inst prefix=`pwd`/inst/usr/local
@@ -2304,7 +2133,7 @@ if test ! -d libogg-${libogg_version}/inst; then
 echo "### Building libogg"
 
 cd libogg-${libogg_version}/
-test -f Makefile || ./configure --enable-shared=no --enable-static=yes $ENABLE_DEBUG $CONFIGURE_OPTIONS
+test -f Makefile || ./configure --enable-shared=no --enable-static=yes $CONFIGURE_OPTIONS
 make
 mkdir -p inst
 make install DESTDIR=`pwd`/inst
@@ -2318,9 +2147,9 @@ echo "### Building libvorbis"
 
 cd libvorbis-${libvorbis_version}/
 if test "$compiler" = "cross-mingw"; then
-test -f Makefile || PKG_CONFIG= ./configure --enable-shared=no --enable-static=yes --with-ogg=$thisdir/libogg-$libogg_version/inst/usr/local $ENABLE_DEBUG $CONFIGURE_OPTIONS
+test -f Makefile || CFLAGS="$CFLAGS -g" PKG_CONFIG= ./configure --enable-shared=no --enable-static=yes --with-ogg=$thisdir/libogg-$libogg_version/inst/usr/local $CONFIGURE_OPTIONS
 else
-test -f Makefile || ./configure --enable-shared=no --enable-static=yes --with-ogg=$thisdir/libogg-$libogg_version/inst/usr/local $ENABLE_DEBUG $CONFIGURE_OPTIONS
+test -f Makefile || CFLAGS="$CFLAGS -g" ./configure --enable-shared=no --enable-static=yes --with-ogg=$thisdir/libogg-$libogg_version/inst/usr/local $CONFIGURE_OPTIONS
 fi
 make
 mkdir -p inst
@@ -2335,7 +2164,7 @@ echo "### Building libflac"
 
 cd flac-${libflac_version}/
 autoreconf -i
-configure_args="--enable-shared=no --enable-static=yes --with-ogg=$thisdir/libogg-$libogg_version/inst/usr/local $ENABLE_DEBUG $CONFIGURE_OPTIONS"
+configure_args="--enable-shared=no --enable-static=yes --with-ogg=$thisdir/libogg-$libogg_version/inst/usr/local --disable-thorough-tests --disable-doxygen-docs --disable-xmms-plugin $FLAC_BUILD_OPTION $CONFIGURE_OPTIONS"
 if test $kernel = "Darwin"; then
   configure_args="$configure_args --disable-asm-optimizations"
 fi
@@ -2343,7 +2172,7 @@ test -f Makefile || ./configure $configure_args
 # On msys32, an error "changed before entering" occurred, can be fixed by
 # modifying /usr/share/perl5/core_perl/File/Path.pm
 # my $Need_Stat_Check = !($^O eq 'MSWin32' || $^O eq 'msys');
-make
+make V=1
 mkdir -p inst
 make install DESTDIR=`pwd`/inst
 cd inst
@@ -2355,12 +2184,15 @@ if test ! -d id3lib-${id3lib_version}/inst; then
 echo "### Building id3lib"
 
 cd id3lib-${id3lib_version}/
+if test $kernel = "MINGW" || test "$compiler" = "cross-mingw"; then
+  sed -i 's/^@ID3_NEEDDEBUG_TRUE@ID3_DEBUG_LIBS = -lcwd -lbfd -liberty$/@ID3_NEEDDEBUG_TRUE@ID3_DEBUG_LIBS =/' examples/Makefile.in
+fi
 autoconf
-configure_args="--enable-shared=no --enable-static=yes $ENABLE_DEBUG $CONFIGURE_OPTIONS"
+configure_args="--enable-shared=no --enable-static=yes $ID3LIB_BUILD_OPTION $CONFIGURE_OPTIONS"
 if test $kernel = "MINGW"; then
   configure_args="$configure_args --build=mingw32"
 fi
-test -f Makefile || CPPFLAGS=-I/usr/local/include LDFLAGS=-L/usr/local/lib ./configure $configure_args
+test -f Makefile || CPPFLAGS=-I/usr/local/include LDFLAGS="$LDFLAGS -L/usr/local/lib" ./configure $configure_args
 SED=sed make
 mkdir -p inst
 make install DESTDIR=`pwd`/inst
@@ -2373,8 +2205,8 @@ if test ! -d taglib-${taglib_version}/inst; then
 echo "### Building taglib"
 
 cd taglib-${taglib_version}/
-test -f Makefile || eval cmake -DWITH_ASF=ON -DWITH_MP4=ON -DINCLUDE_DIRECTORIES=/usr/local/include -DLINK_DIRECTORIES=/usr/local/lib -DENABLE_STATIC=ON -DZLIB_ROOT=../zlib-$zlib_version/inst/usr/local $CMAKE_BUILD_TYPE_DEBUG $CMAKE_OPTIONS
-make
+test -f Makefile || eval cmake -DWITH_ASF=ON -DWITH_MP4=ON -DINCLUDE_DIRECTORIES=/usr/local/include -DLINK_DIRECTORIES=/usr/local/lib $taglib_static_option -DZLIB_ROOT=../zlib-$zlib_version/inst/usr/local $CMAKE_BUILD_OPTION $CMAKE_OPTIONS
+make VERBOSE=1
 mkdir -p inst
 make install DESTDIR=`pwd`/inst
 fixcmakeinst
@@ -2400,11 +2232,7 @@ if test "$compiler" = "cross-mingw"; then
 sed -i 's/^\(.*-Werror=missing-prototypes\)/#\1/' ./configure
 AV_CONFIGURE_OPTIONS="--cross-prefix=${cross_host}- --arch=x86 --target-os=mingw32 --sysinclude=/usr/${cross_host}/include"
 fi
-if test -z "$ENABLE_DEBUG"; then
-AV_CONFIGURE_OPTIONS="$AV_CONFIGURE_OPTIONS --disable-debug"
-else
-AV_CONFIGURE_OPTIONS="$AV_CONFIGURE_OPTIONS --enable-debug=3"
-fi
+AV_CONFIGURE_OPTIONS="$AV_CONFIGURE_OPTIONS $AV_BUILD_OPTION"
 ./configure \
 	--enable-memalign-hack \
 	--disable-shared \
@@ -2513,11 +2341,7 @@ fi
 if ( test $kernel = "Darwin" || test $kernel = "MINGW" ) && test -n "${ffmpeg_version}"; then
 AV_CONFIGURE_OPTIONS="$AV_CONFIGURE_OPTIONS --disable-iconv"
 fi
-if test -z "$ENABLE_DEBUG"; then
-AV_CONFIGURE_OPTIONS="$AV_CONFIGURE_OPTIONS --disable-debug"
-else
-AV_CONFIGURE_OPTIONS="$AV_CONFIGURE_OPTIONS --enable-debug=3"
-fi
+AV_CONFIGURE_OPTIONS="$AV_CONFIGURE_OPTIONS $AV_BUILD_OPTION"
 ./configure \
 	--enable-memalign-hack \
 	--disable-shared \
@@ -2595,7 +2419,7 @@ fi
 	--enable-decoder=rawvideo \
 	--disable-hwaccel=h264_dxva2 \
 	--disable-hwaccel=mpeg2_dxva2 $AV_CONFIGURE_OPTIONS
-make
+make V=1
 mkdir -p inst
 make install DESTDIR=`pwd`/inst
 cd inst
@@ -2609,7 +2433,8 @@ echo "### Building chromaprint"
 
 # The zlib library path was added for MinGW-builds GCC 4.7.2.
 cd chromaprint-${chromaprint_version}/
-test -f Makefile || eval cmake -DBUILD_SHARED_LIBS=OFF -DEXTRA_LIBS=\"-L$thisdir/zlib-$zlib_version/inst/usr/local/lib -lz\" -DFFMPEG_ROOT=$thisdir/$ffmpeg_dir/inst/usr/local $CMAKE_BUILD_TYPE_DEBUG $CMAKE_OPTIONS
+test -f Makefile || eval cmake -DBUILD_SHARED_LIBS=OFF -DEXTRA_LIBS=\"-L$thisdir/zlib-$zlib_version/inst/usr/local/lib -lz\" -DFFMPEG_ROOT=$thisdir/$ffmpeg_dir/inst/usr/local $CMAKE_BUILD_OPTION $CMAKE_OPTIONS
+make VERBOSE=1
 mkdir -p inst
 make install DESTDIR=`pwd`/inst
 fixcmakeinst
@@ -2626,7 +2451,7 @@ if test $kernel = "MINGW" || test "$compiler" = "cross-mingw" ||
    test $kernel = "Darwin"; then
 autoreconf -i
 fi
-test -f Makefile || CXXFLAGS="-g -O2 -DMP4V2_USE_STATIC_LIB" ./configure --enable-shared=no --enable-static=yes --disable-gch $CONFIGURE_OPTIONS
+test -f Makefile || CXXFLAGS="$CXXFLAGS -g -O2 -DMP4V2_USE_STATIC_LIB" ./configure --enable-shared=no --enable-static=yes --disable-gch $CONFIGURE_OPTIONS
 mkdir -p inst
 make install DESTDIR=`pwd`/inst
 cd inst
@@ -2646,7 +2471,7 @@ if test $kernel = "Linux"; then
     mkdir kid3
     if test "$compiler" = "cross-mingw"; then
       cat >kid3/build.sh <<EOF
-cmake -DCMAKE_BUILD_TYPE=Release -DCMAKE_TOOLCHAIN_FILE=$thisdir/source/mingw.cmake -DCMAKE_INSTALL_PREFIX= -DWITH_FFMPEG=ON -DWITH_MP4V2=ON -DCMAKE_CXX_FLAGS="-g -O2 -DMP4V2_USE_STATIC_LIB" -DDOCBOOK_XSL_DIR=/usr/share/xml/docbook/stylesheet/nwalsh ../../kid3
+cmake $CMAKE_BUILD_OPTION -DCMAKE_TOOLCHAIN_FILE=$thisdir/source/mingw.cmake -DCMAKE_INSTALL_PREFIX= -DWITH_FFMPEG=ON -DWITH_MP4V2=ON -DCMAKE_CXX_FLAGS="-g -O2 -DMP4V2_USE_STATIC_LIB" -DDOCBOOK_XSL_DIR=/usr/share/xml/docbook/stylesheet/nwalsh ../../kid3
 EOF
       cat >kid3/make_package.sh <<"EOF"
 #!/bin/sh
@@ -2679,11 +2504,11 @@ EOF
       chmod +x kid3/make_package.sh
     else
       taglib_config_version=$taglib_version
-      test $taglib_config_version = "1.10beta" && taglib_config_version="1.10.0"
+      taglib_config_version=${taglib_config_version%beta*}
       cat >kid3/build.sh <<EOF
 BUILDPREFIX=\$(cd ..; pwd)/buildroot/usr/local
 export PKG_CONFIG_PATH=\$BUILDPREFIX/lib/pkgconfig
-cmake -DWITH_TAGLIB=OFF -DHAVE_TAGLIB=1 -DTAGLIB_LIBRARIES:STRING="-L\$BUILDPREFIX/lib -ltag -lz" -DTAGLIB_CFLAGS:STRING="-I\$BUILDPREFIX/include/taglib -I\$BUILDPREFIX/include -DTAGLIB_STATIC" -DTAGLIB_VERSION:STRING="${taglib_config_version}" -DWITH_QT5=ON -DWITH_QML=OFF -DCMAKE_CXX_FLAGS_DEBUG:STRING="-g -DID3LIB_LINKOPTION=1 -DFLAC__NO_DLL" -DCMAKE_INCLUDE_PATH=\$BUILDPREFIX/include -DCMAKE_LIBRARY_PATH=\$BUILDPREFIX/lib -DCMAKE_PROGRAM_PATH=\$BUILDPREFIX/bin -DWITH_FFMPEG=ON -DFFMPEG_ROOT=\$BUILDPREFIX -DWITH_MP4V2=ON -DCMAKE_BUILD_TYPE=Debug -DBUILD_SHARED_LIBS=OFF -DWITH_GCC_PCH=OFF -DWITH_APPS="Qt;CLI" -DCMAKE_INSTALL_PREFIX= -DWITH_BINDIR=. -DWITH_DATAROOTDIR=. -DWITH_DOCDIR=. -DWITH_TRANSLATIONSDIR=. ../../kid3
+cmake -DWITH_TAGLIB=OFF -DHAVE_TAGLIB=1 -DTAGLIB_LIBRARIES:STRING="-L\$BUILDPREFIX/lib -ltag -lz" -DTAGLIB_CFLAGS:STRING="-I\$BUILDPREFIX/include/taglib -I\$BUILDPREFIX/include -DTAGLIB_STATIC" -DTAGLIB_VERSION:STRING="${taglib_config_version}" -DWITH_QT5=ON -DWITH_QML=OFF -DCMAKE_CXX_FLAGS_DEBUG:STRING="-g -DID3LIB_LINKOPTION=1 -DFLAC__NO_DLL" -DCMAKE_INCLUDE_PATH=\$BUILDPREFIX/include -DCMAKE_LIBRARY_PATH=\$BUILDPREFIX/lib -DCMAKE_PROGRAM_PATH=\$BUILDPREFIX/bin -DWITH_FFMPEG=ON -DFFMPEG_ROOT=\$BUILDPREFIX -DWITH_MP4V2=ON $CMAKE_BUILD_OPTION -DBUILD_SHARED_LIBS=OFF -DWITH_GCC_PCH=OFF -DWITH_APPS="Qt;CLI" -DCMAKE_INSTALL_PREFIX= -DWITH_BINDIR=. -DWITH_DATAROOTDIR=. -DWITH_DOCDIR=. -DWITH_TRANSLATIONSDIR=. ../../kid3
 EOF
     fi
     chmod +x kid3/build.sh
