@@ -162,14 +162,14 @@ Frame::TagVersion CliCommand::getTagMaskParameter(int nr,
         tagMask |= 1;
       if (tagStr.contains(QLatin1Char('2')))
         tagMask |= 2;
-      if (tagStr == QLatin1String("3"))
-        tagMask = 3;
+      if (tagMask == 0)
+        tagMask = tagStr.toInt();
     }
   }
   if (tagMask == 0 && useDefault) {
     tagMask = m_processor->tagMask();
   }
-  return TrackData::tagVersionCast(tagMask);
+  return Frame::tagVersionCast(tagMask);
 }
 
 /**
@@ -404,14 +404,12 @@ void GetCommand::startCommand()
   if (name == QLatin1String("all")) {
     cli()->writeFileInformation(tagMask);
   } else {
-    foreach (Frame::TagVersion tagBit, QList<Frame::TagVersion>()
-             << Frame::TagV2 << Frame::TagV1) {
-      if (tagMask & tagBit) {
-        QString value = cli()->app()->getFrame(tagBit, name);
-        if (!value.isEmpty()) {
-          cli()->writeLine(value);
-          break;
-        }
+    foreach (Frame::TagNumber tagNr, Frame::tagNumbersFromMask(tagMask)) {
+      QString value = cli()->app()->getFrame(Frame::tagVersionFromNumber(tagNr),
+                                             name);
+      if (!(tagNr == Frame::Tag_1 ? value.isEmpty() : value.isNull())) {
+        cli()->writeLine(value);
+        break;
       }
     }
   }
@@ -458,14 +456,24 @@ void RevertCommand::startCommand()
 ImportCommand::ImportCommand(Kid3Cli* processor) :
   CliCommand(processor, QLatin1String("import"),
              tr("Import from file or clipboard"),
-             QLatin1String("P S [T]\nS = ") + tr("Format name"))
+             QLatin1String("P S [T]\nP S = ") +
+             tr("File path") + QLatin1Char(' ') + tr("Format name") +
+             QLatin1String(" | ") +
+             QLatin1String("clipboard ") + tr("Format name") +
+             QLatin1String("\n    | tags ") + tr("Source") + QLatin1Char(' ') +
+             tr("Extraction"))
 {
 }
 
 void ImportCommand::startCommand()
 {
   int numArgs = args().size();
-  if (numArgs > 2) {
+  if (numArgs > 3 && args().at(1) == QLatin1String("tags")) {
+    const QString& source = args().at(2);
+    const QString& extraction = args().at(3);
+    Frame::TagVersion tagMask = getTagMaskParameter(4);
+    cli()->app()->importFromTags(tagMask, source, extraction);
+  } else if (numArgs > 2) {
     const QString& path = args().at(1);
     const QString& fmtName = args().at(2);
     bool ok;
@@ -821,7 +829,10 @@ void NumberTracksCommand::startCommand()
     firstTrackNr = 1;
   }
   Frame::TagVersion tagMask = getTagMaskParameter(2);
-  cli()->app()->numberTracks(firstTrackNr, 0, tagMask);
+  Kid3Application::NumberTrackOptions options;
+  options |= Kid3Application::NumberTracksEnabled;
+  options |= Kid3Application::NumberTracksResetCounterForEachDirectory;
+  cli()->app()->numberTracks(firstTrackNr, 0, tagMask, options);
 }
 
 
