@@ -86,12 +86,6 @@ public:
    */
   bool selectFile(const QStringList &paths);
 
-  /**
-   * Get indexes of selected files.
-   * @return selected indexes.
-   */
-  QList<QPersistentModelIndex> getSelection() const;
-
   /**
    * Display help about available commands.
    * @param cmdName command name, for all commands if empty
@@ -118,14 +112,14 @@ public:
    * Get currently active tag mask.
    * @return tag bits.
    */
-  TrackData::TagVersion tagMask() const { return m_tagMask; }
+  Frame::TagVersion tagMask() const { return m_tagMask; }
 
   /**
    * Set currently active tag mask.
    *
    * @param tagMask tag bits
    */
-  void setTagMask(TrackData::TagVersion tagMask);
+  void setTagMask(Frame::TagVersion tagMask);
 
   /**
    * Get timeout value.
@@ -169,11 +163,8 @@ private slots:
   /**
    * Select files passed as command line arguments after the initial directory has
    * been opened. Start execution of commands if existing.
-   * @param dirIndex file proxy model index of opened directory
-   * @param fileIndexes file proxy model indexes of selected files
    */
-  void onInitialDirectoryOpened(const QPersistentModelIndex& dirIndex,
-                               const QList<QPersistentModelIndex>& fileIndexes);
+  void onInitialDirectoryOpened();
 
   /**
    * Called when a command is finished.
@@ -204,12 +195,10 @@ private:
 #endif
   QList<CliCommand*> m_cmds;
   QStringList m_argCommands;
-  TaggedFile::DetailInfo m_detailInfo;
+  QString m_detailInfo;
   QString m_filename;
-  QString m_tagFormatV1;
-  QString m_tagFormatV2;
-  TrackData::TagVersion m_tagMask;
-  QList<QPersistentModelIndex> m_selection;
+  QString m_tagFormat[Frame::Tag_NumValues];
+  Frame::TagVersion m_tagMask;
   /** Overwrites command timeout, -1 to switch off, 0 for defaults, else ms. */
   int m_timeoutMs;
   bool m_fileNameChanged;
