@@ -30,6 +30,7 @@
 #include <QFile>
 #include <QTextStream>
 #include "playlistconfig.h"
+#include "fileconfig.h"
 #include "taggedfile.h"
 #include "trackdata.h"
 #include "fileproxymodel.h"
@@ -45,10 +46,10 @@ PlaylistCreator::PlaylistCreator(const QString& topLevelDir,
                                  const PlaylistConfig& cfg) :
   m_cfg(cfg)
 {
-  if (m_cfg.m_location == PlaylistConfig::PL_TopLevelDirectory) {
+  if (m_cfg.location() == PlaylistConfig::PL_TopLevelDirectory) {
     m_playlistDirName = topLevelDir;
-    if (!m_playlistDirName.endsWith(QChar(QDir::separator()))) {
-      m_playlistDirName += QDir::separator();
+    if (!m_playlistDirName.endsWith(QLatin1Char('/'))) {
+      m_playlistDirName += QLatin1Char('/');
     }
   }
 }
@@ -66,16 +67,20 @@ bool PlaylistCreator::write()
     ok = file.open(QIODevice::WriteOnly);
     if (ok) {
       QTextStream stream(&file);
+      QString codecName = FileConfig::instance().textEncoding();
+      if (codecName != QLatin1String("System")) {
+        stream.setCodec(codecName.toLatin1());
+      }
 
-      switch (m_cfg.m_format) {
+      switch (m_cfg.format()) {
         case PlaylistConfig::PF_M3U:
-          if (m_cfg.m_writeInfo) {
+          if (m_cfg.writeInfo()) {
             stream << "#EXTM3U\n";
           }
           for (QMap<QString, Entry>::const_iterator it = m_entries.begin();
                it != m_entries.end();
                ++it) {
-            if (m_cfg.m_writeInfo) {
+            if (m_cfg.writeInfo()) {
               stream << QString(QLatin1String("#EXTINF:%1,%2\n")).
                 arg((*it).duration).arg((*it).info);
             }
@@ -91,7 +96,7 @@ bool PlaylistCreator::write()
                it != m_entries.end();
                ++it) {
             stream << QString(QLatin1String("File%1=%2\n")).arg(nr).arg((*it).filePath);
-            if (m_cfg.m_writeInfo) {
+            if (m_cfg.writeInfo()) {
               stream << QString(QLatin1String("Title%1=%2\n")).arg(nr).arg((*it).info);
               stream << QString(QLatin1String("Length%1=%2\n")).arg(nr).arg((*it).duration);
             }
@@ -104,7 +109,7 @@ bool PlaylistCreator::write()
         {
           stream << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
           QString line = QLatin1String("<playlist version=\"1\" xmlns=\"http://xspf.org/ns/0/\"");
-          if (!m_cfg.m_useFullPath) {
+          if (!m_cfg.useFullPath()) {
             QUrl url(m_playlistDirName);
             url.setScheme(QLatin1String("file"));
             line += QString(QLatin1String(" xml:base=\"%1\"")).arg(QString::fromLatin1(url.toEncoded().data()));
@@ -118,12 +123,12 @@ bool PlaylistCreator::write()
                ++it) {
             stream << "    <track>\n";
             QUrl url((*it).filePath);
-            if (m_cfg.m_useFullPath) {
+            if (m_cfg.useFullPath()) {
               url.setScheme(QLatin1String("file"));
             }
             stream << QString(QLatin1String("      <location>%1</location>\n")).arg(
               QString::fromLatin1(url.toEncoded().data()));
-            if (m_cfg.m_writeInfo) {
+            if (m_cfg.writeInfo()) {
               // the info is already formatted in the case of XSPF
               stream << (*it).info;
             }
@@ -161,12 +166,11 @@ PlaylistCreator::Item::Item(const QModelIndex& index, PlaylistCreator& ctr) :
     m_dirName = FileProxyModel::getPathIfIndexOfDir(index);
     m_isDir = !m_dirName.isNull();
   }
-  QChar separator = QDir::separator();
-  if (!m_dirName.endsWith(separator)) {
-    m_dirName += separator;
+  if (!m_dirName.endsWith(QLatin1Char('/'))) {
+    m_dirName += QLatin1Char('/');
   }
   // fix double separators
-  m_dirName.replace(QString(separator) + separator, separator);
+  m_dirName.replace(QLatin1String("//"), QLatin1String("/"));
 }
 
 /**
@@ -187,9 +191,8 @@ PlaylistCreator::Item::~Item()
 QString PlaylistCreator::Item::formatString(const QString& format)
 {
   if (!m_trackData) {
-    m_taggedFile->readTags(false);
-    m_taggedFile = FileProxyModel::readWithId3V24IfId3V24(m_taggedFile);
-    m_trackData = new ImportTrackData(*m_taggedFile, ImportTrackData::TagV2V1);
+    m_taggedFile = FileProxyModel::readTagsFromTaggedFile(m_taggedFile);
+    m_trackData = new ImportTrackData(*m_taggedFile, Frame::TagVAll);
   }
   return m_trackData->formatString(format);
 }
@@ -204,19 +207,19 @@ QString PlaylistCreator::Item::formatString(const QString& format)
 bool PlaylistCreator::Item::add()
 {
   bool ok = true;
-  if (m_ctr.m_cfg.m_location != PlaylistConfig::PL_TopLevelDirectory) {
+  if (m_ctr.m_cfg.location() != PlaylistConfig::PL_TopLevelDirectory) {
     if (m_ctr.m_playlistDirName != m_dirName) {
       ok = m_ctr.write();
       m_ctr.m_playlistDirName = m_dirName;
     }
   }
   if (m_ctr.m_playlistFileName.isEmpty()) {
-    if (!m_ctr.m_cfg.m_useFileNameFormat) {
+    if (!m_ctr.m_cfg.useFileNameFormat()) {
       m_ctr.m_playlistFileName = QDir(m_ctr.m_playlistDirName).dirName();
     } else {
-      m_ctr.m_playlistFileName = formatString(m_ctr.m_cfg.m_fileNameFormat);
+      m_ctr.m_playlistFileName = formatString(m_ctr.m_cfg.fileNameFormat());
     }
-    switch (m_ctr.m_cfg.m_format) {
+    switch (m_ctr.m_cfg.format()) {
       case PlaylistConfig::PF_M3U:
         m_ctr.m_playlistFileName += QLatin1String(".m3u");
         break;
@@ -229,20 +232,20 @@ bool PlaylistCreator::Item::add()
     }
   }
   QString filePath = m_dirName + m_taggedFile->getFilename();
-  if (!m_ctr.m_cfg.m_useFullPath &&
+  if (!m_ctr.m_cfg.useFullPath() &&
       filePath.startsWith(m_ctr.m_playlistDirName)) {
     filePath = filePath.mid(m_ctr.m_playlistDirName.length());
   }
   QString sortKey;
-  if (m_ctr.m_cfg.m_useSortTagField) {
-    sortKey = formatString(m_ctr.m_cfg.m_sortTagField);
+  if (m_ctr.m_cfg.useSortTagField()) {
+    sortKey = formatString(m_ctr.m_cfg.sortTagField());
   }
   sortKey += filePath;
   PlaylistCreator::Entry entry;
   entry.filePath = filePath;
-  if (m_ctr.m_cfg.m_writeInfo) {
-    if (m_ctr.m_cfg.m_format != PlaylistConfig::PF_XSPF) {
-      entry.info = formatString(m_ctr.m_cfg.m_infoFormat);
+  if (m_ctr.m_cfg.writeInfo()) {
+    if (m_ctr.m_cfg.format() != PlaylistConfig::PF_XSPF) {
+      entry.info = formatString(m_ctr.m_cfg.infoFormat());
     } else {
       entry.info = formatString(QLatin1String(
         "      <title>%{title}</title>\n"
