@@ -148,8 +148,9 @@ const char* getNameFromType(Frame::Type type)
     QT_TRANSLATE_NOOP("@default", "Subtitle"),        // FT_Subtitle,
     QT_TRANSLATE_NOOP("@default", "Website"),         // FT_Website,
     QT_TRANSLATE_NOOP("@default", "WWW Audio File"),  // FT_WWWAudioFile,
-    QT_TRANSLATE_NOOP("@default", "WWW Audio Source") // FT_WWWAudioSource,
-                                  // FT_LastFrame = FT_WWWAudioSource
+    QT_TRANSLATE_NOOP("@default", "WWW Audio Source"), // FT_WWWAudioSource,
+    QT_TRANSLATE_NOOP("@default", "Release Date")     // FT_ReleaseDate,
+                                  // FT_LastFrame = FT_ReleaseDate
   };
   struct not_used { int array_size_check[
       sizeof(names) / sizeof(names[0]) == Frame::FT_LastFrame + 1
@@ -299,6 +300,36 @@ int Frame::numberWithoutTotal(const QString& str, bool* ok)
     str.left(slashPos).toInt(ok);
 }
 
+/**
+ * Get value as integer.
+ * @return value.
+ */
+int Frame::getValueAsNumber() const
+{
+  if (isInactive()) {
+    return -1;
+  } else if (isEmpty()) {
+    return 0;
+  } else {
+    return numberWithoutTotal(m_value);
+  }
+}
+
+/**
+ * Set value as integer.
+ * @param n value as number
+ */
+void Frame::setValueAsNumber(int n)
+{
+  if (n == -1) {
+    m_value = QString();
+  } else if (n == 0) {
+    m_value = QLatin1String("");
+  } else {
+    m_value.setNum(n);
+  }
+}
+
 /**
  * Get the value of a field.
  *
@@ -541,7 +572,6 @@ QString Frame::getDisplayName(const QString& name)
     { "PRODUCER", QT_TRANSLATE_NOOP("@default", "Producer") },
     { "PRODUCTNUMBER", QT_TRANSLATE_NOOP("@default", "Product Number") },
     { "RECORDINGDATE", QT_TRANSLATE_NOOP("@default", "Recording Date") },
-    { "RELEASEDATE", QT_TRANSLATE_NOOP("@default", "Release Date") },
     { "REMIXEDBY", QT_TRANSLATE_NOOP("@default", "Remixer") },
     { "Rating", QT_TRANSLATE_NOOP("@default", "Rating") },
     { "TOTALDISCS", QT_TRANSLATE_NOOP("@default", "Total Discs") },
@@ -585,7 +615,18 @@ QString Frame::getDisplayName(const QString& name)
     { "tvnn", QT_TRANSLATE_NOOP("@default", "TV Network Name") },
     { "tvsh", QT_TRANSLATE_NOOP("@default", "TV Show Name") },
     { "tvsn", QT_TRANSLATE_NOOP("@default", "TV Season") },
-    { "year", QT_TRANSLATE_NOOP("@default", "Year") }
+    { "year", QT_TRANSLATE_NOOP("@default", "Year") },
+    { "IARL", QT_TRANSLATE_NOOP("@default", "Archival Location") },
+    { "ICMS", QT_TRANSLATE_NOOP("@default", "Commissioned") },
+    { "ICRP", QT_TRANSLATE_NOOP("@default", "Cropped") },
+    { "IDIM", QT_TRANSLATE_NOOP("@default", "Dimensions") },
+    { "IDPI", QT_TRANSLATE_NOOP("@default", "Dots Per Inch") },
+    { "IKEY", QT_TRANSLATE_NOOP("@default", "Keywords") },
+    { "ILGT", QT_TRANSLATE_NOOP("@default", "Lightness") },
+    { "IPLT", QT_TRANSLATE_NOOP("@default", "Number of Colors") },
+    { "ISBJ", QT_TRANSLATE_NOOP("@default", "Subject") },
+    { "ISHP", QT_TRANSLATE_NOOP("@default", "Sharpness") },
+    { "ISRF", QT_TRANSLATE_NOOP("@default", "Source Form") }
   };
   static QMap<QByteArray, QByteArray> idStrMap;
   if (idStrMap.isEmpty()) {
@@ -742,6 +783,53 @@ const char* const* Frame::Field::getContentTypeNames()
   return contentTypeNames;
 }
 
+/**
+ * Get list of available tag versions with translated description.
+ * @return tag version/description pairs.
+ */
+QList<QPair<Frame::TagVersion, QString> > Frame::availableTagVersions()
+{
+  QList<QPair<TagVersion, QString> > result;
+  FOR_ALL_TAGS(tagNr) {
+    const char* const tagStr = QT_TRANSLATE_NOOP("@default", "Tag %1");
+    result << qMakePair(Frame::tagVersionCast(1 << tagNr),
+                        QCoreApplication::translate("@default", tagStr)
+                        .arg(Frame::tagNumberToString(tagNr)));
+  }
+  const char* const tag12Str = QT_TRANSLATE_NOOP("@default", "Tag 1 and Tag 2");
+  result << qMakePair(TagV2V1, QCoreApplication::translate("@default",
+                                                           tag12Str));
+  if (TagVAll != TagV2V1) {
+    const char* const allTagsStr = QT_TRANSLATE_NOOP("@default", "All Tags");
+    result << qMakePair(TagVAll, QCoreApplication::translate("@default",
+                                                             allTagsStr));
+  }
+  return result;
+}
+
+/**
+ * Get string representation for tag number.
+ * @param tagNr tag number
+ * @return "1" for Tag_1, "2" for Tag_2, ..., null if invalid.
+ */
+QString Frame::tagNumberToString(TagNumber tagNr)
+{
+  return tagNr < Tag_NumValues ? QString::number(tagNr + 1) : QString();
+}
+
+/**
+ * Get tag number from string representation.
+ * @param str string representation
+ * @return Tag_1 for "1", Tag_2 for "2", ..., Tag_NumValues if invalid.
+ */
+Frame::TagNumber Frame::tagNumberFromString(const QString& str)
+{
+  bool ok;
+  int nr = str.toInt(&ok);
+  return ok && --nr >= Frame::Tag_1 && nr < Frame::Tag_NumValues
+      ? static_cast<Frame::TagNumber>(nr) : Tag_NumValues;
+}
+
 #ifndef QT_NO_DEBUG
 /**
  * Get string representation of variant.
@@ -891,6 +979,17 @@ void FrameCollection::removeDisabledFrames(const FrameFilter& flt)
   }
 }
 
+/**
+ * Set the index of all frames to -1.
+ */
+void FrameCollection::setIndexesInvalid()
+{
+  for (iterator it = begin(); it != end(); ++it) {
+    Frame& frame = const_cast<Frame&>(*it);
+    frame.setIndex(-1);
+  }
+}
+
 /**
  * Copy frames which are empty or inactive from other frames.
  * This can be used to merge two frame collections.
