@@ -37,8 +37,10 @@
 #include <QSpinBox>
 #include <QVBoxLayout>
 #include <QHBoxLayout>
+#include <QFormLayout>
 #include <QGroupBox>
 #include <QStringListModel>
+#include <QStandardItemModel>
 #include "formatconfig.h"
 #include "formatbox.h"
 #include "tagconfig.h"
@@ -51,15 +53,20 @@
 #include "configtable.h"
 #include "commandstablemodel.h"
 #include "checkablestringlistmodel.h"
+#include "fileproxymodel.h"
 #include "contexthelp.h"
 #include "frame.h"
+#include "iplatformtools.h"
 
 /**
  * Constructor.
  */
-ConfigDialogPages::ConfigDialogPages(QObject* parent) : QObject(parent),
+ConfigDialogPages::ConfigDialogPages(IPlatformTools* platformTools,
+                                     QObject* parent) : QObject(parent),
+  m_platformTools(platformTools),
   m_loadLastOpenedFileCheckBox(0), m_preserveTimeCheckBox(0),
   m_markChangesCheckBox(0), m_coverFileNameLineEdit(0),
+  m_nameFilterComboBox(0), m_fileTextEncodingComboBox(0),
   m_markTruncationsCheckBox(0), m_textEncodingV1ComboBox(0),
   m_totalNumTracksCheckBox(0), m_commentNameComboBox(0),
   m_pictureNameComboBox(0), m_markOversizedPicturesCheckBox(0),
@@ -99,7 +106,7 @@ QWidget* ConfigDialogPages::createTagsPage()
   v1GroupBoxLayout->addWidget(m_markTruncationsCheckBox, 0, 0, 1, 2);
   QLabel* textEncodingV1Label = new QLabel(tr("Text &encoding:"), v1GroupBox);
   m_textEncodingV1ComboBox = new QComboBox(v1GroupBox);
-  m_textEncodingV1ComboBox->addItems(TagConfig::getTextEncodingV1Names());
+  m_textEncodingV1ComboBox->addItems(TagConfig::getTextCodecNames());
   m_textEncodingV1ComboBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
   textEncodingV1Label->setBuddy(m_textEncodingV1ComboBox);
   v1GroupBoxLayout->addWidget(textEncodingV1Label, 1, 0);
@@ -196,18 +203,22 @@ QWidget* ConfigDialogPages::createTagsPage()
   genresGroupBox->setLayout(vbox);
   tag2RightLayout->addWidget(genresGroupBox);
 
-  QGroupBox* quickAccessTagsGroupBox = new QGroupBox(tr("&Quick Access Tags"));
+  QGroupBox* quickAccessTagsGroupBox = new QGroupBox(tr("&Quick Access Frames"));
   QVBoxLayout* quickAccessTagsLayout = new QVBoxLayout(quickAccessTagsGroupBox);
   QListView* quickAccessTagsListView = new QListView;
-  m_quickAccessTagsModel = new CheckableStringListModel(quickAccessTagsGroupBox);
-  QStringList unifiedFrameNames;
-  for (int i = Frame::FT_FirstFrame; i< Frame::FT_LastFrame; ++i) {
-    unifiedFrameNames.append(
-        Frame::ExtendedType(static_cast<Frame::Type>(i)).getTranslatedName());
-  }
-  m_quickAccessTagsModel->setStringList(unifiedFrameNames);
+  m_quickAccessTagsModel = new QStandardItemModel(quickAccessTagsGroupBox);
   quickAccessTagsListView->setModel(m_quickAccessTagsModel);
+  quickAccessTagsListView->setAcceptDrops(true);
+  quickAccessTagsListView->setDragEnabled(true);
+  quickAccessTagsListView->setDragDropMode(QAbstractItemView::InternalMove);
+  quickAccessTagsListView->setDragDropOverwriteMode(false);
+  quickAccessTagsListView->setDefaultDropAction(Qt::MoveAction);
+  quickAccessTagsListView->setDropIndicatorShown(true);
   quickAccessTagsLayout->addWidget(quickAccessTagsListView);
+  QLabel* reorderLabel =
+      new QLabel(tr("Use drag and drop to reorder the items"));
+  reorderLabel->setWordWrap(true);
+  quickAccessTagsLayout->addWidget(reorderLabel);
   tag2RightLayout->addWidget(quickAccessTagsGroupBox);
   tag2Layout->addLayout(tag2RightLayout);
 
@@ -236,29 +247,68 @@ QWidget* ConfigDialogPages::createFilesPage()
 {
   QWidget* filesPage = new QWidget;
   QVBoxLayout* vlayout = new QVBoxLayout(filesPage);
+  QHBoxLayout* hlayout = new QHBoxLayout;
+  QVBoxLayout* leftLayout = new QVBoxLayout;
+  QVBoxLayout* rightLayout = new QVBoxLayout;
+
   QGroupBox* startupGroupBox = new QGroupBox(tr("Startup"), filesPage);
   m_loadLastOpenedFileCheckBox = new QCheckBox(tr("&Load last-opened files"),
                                                startupGroupBox);
   QVBoxLayout* startupLayout = new QVBoxLayout;
   startupLayout->addWidget(m_loadLastOpenedFileCheckBox);
   startupGroupBox->setLayout(startupLayout);
-  vlayout->addWidget(startupGroupBox);
+  leftLayout->addWidget(startupGroupBox);
   QGroupBox* saveGroupBox = new QGroupBox(tr("Save"), filesPage);
   m_preserveTimeCheckBox = new QCheckBox(tr("&Preserve file timestamp"), saveGroupBox);
   m_markChangesCheckBox = new QCheckBox(tr("&Mark changes"), saveGroupBox);
-  QLabel* coverFileNameLabel = new QLabel(tr("F&ilename for cover:"), saveGroupBox);
   m_coverFileNameLineEdit = new QLineEdit(saveGroupBox);
-  coverFileNameLabel->setBuddy(m_coverFileNameLineEdit);
-  QHBoxLayout* hbox = new QHBoxLayout;
-  hbox->setContentsMargins(2, 0, 2, 0);
-  hbox->addWidget(coverFileNameLabel);
-  hbox->addWidget(m_coverFileNameLineEdit);
-  QVBoxLayout* vbox = new QVBoxLayout;
-  vbox->addWidget(m_preserveTimeCheckBox);
-  vbox->addWidget(m_markChangesCheckBox);
-  vbox->addLayout(hbox);
-  saveGroupBox->setLayout(vbox);
-  vlayout->addWidget(saveGroupBox);
+  m_fileTextEncodingComboBox = new QComboBox(saveGroupBox);
+  m_fileTextEncodingComboBox->addItems(FileConfig::getTextCodecNames());
+  m_fileTextEncodingComboBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,
+                                                        QSizePolicy::Minimum));
+  QFormLayout* formLayout = new QFormLayout;
+  formLayout->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
+  formLayout->addRow(m_preserveTimeCheckBox);
+  formLayout->addRow(m_markChangesCheckBox);
+  formLayout->addRow(tr("F&ilename for cover:"), m_coverFileNameLineEdit);
+  formLayout->addRow(tr("Text &encoding (Export, Playlist):"),
+                     m_fileTextEncodingComboBox);
+  saveGroupBox->setLayout(formLayout);
+  leftLayout->addWidget(saveGroupBox);
+
+  QGroupBox* fileListGroupBox = new QGroupBox(tr("File List"), filesPage);
+  QLabel* nameFilterLabel = new QLabel(tr("Filte&r:"), fileListGroupBox);
+  m_nameFilterComboBox = new QComboBox(fileListGroupBox);
+  m_nameFilterComboBox->setSizePolicy(QSizePolicy(QSizePolicy::Expanding,
+                                                  QSizePolicy::Minimum));
+  QList<QPair<QString, QString> > filters = FileProxyModel::createNameFilters();
+  for (QList<QPair<QString, QString> >::const_iterator it = filters.constBegin();
+       it != filters.constEnd();
+       ++it) {
+    QString nameFilter = m_platformTools->fileDialogNameFilter(
+          QList<QPair<QString, QString> >() << *it);
+#if QT_VERSION < 0x050000
+    // Filters from the KDE 4 file dialog are not consistent, filter uses
+    // something like "*.mp3|MP3 (*.mp3)", but the current filter returns only
+    // something like "*.mp3".
+    int pipeIndex = nameFilter.indexOf(QLatin1Char('|'));
+    if (pipeIndex != -1) {
+      nameFilter = nameFilter.left(pipeIndex);
+    }
+#endif
+    m_nameFilterComboBox->addItem(it->first, nameFilter);
+  }
+  nameFilterLabel->setBuddy(m_nameFilterComboBox);
+  QGridLayout* fileListGroupBoxLayout = new QGridLayout(fileListGroupBox);
+  fileListGroupBoxLayout->addWidget(nameFilterLabel, 0, 0);
+  fileListGroupBoxLayout->addWidget(m_nameFilterComboBox, 0, 1);
+  rightLayout->addWidget(fileListGroupBox);
+  rightLayout->addStretch();
+
+  hlayout->addLayout(leftLayout);
+  hlayout->addLayout(rightLayout);
+  vlayout->addLayout(hlayout);
+
   QString fnFormatTitle(tr("&Filename Format"));
   m_fnFormatBox = new FormatBox(fnFormatTitle, filesPage);
   m_fnFormatBox->hideValidationCheckBox();
@@ -374,6 +424,29 @@ QWidget* ConfigDialogPages::createPluginsPage()
   return pluginsPage;
 }
 
+/**
+ * Set values in pages from default configuration.
+ */
+void ConfigDialogPages::setDefaultConfig()
+{
+  FilenameFormatConfig fnCfg;
+  TagFormatConfig id3Cfg;
+  TagConfig tagCfg;
+  tagCfg.setAvailablePlugins(TagConfig::instance().availablePlugins());
+  tagCfg.setTaggedFileFeatures(TagConfig::instance().taggedFileFeatures());
+  tagCfg.setDefaultPluginOrder();
+  FileConfig fileCfg;
+  UserActionsConfig userActionsCfg;
+  userActionsCfg.setDefaultUserActions();
+  GuiConfig guiCfg;
+  NetworkConfig networkCfg;
+  networkCfg.setDefaultBrowser();
+  ImportConfig importCfg;
+  importCfg.setAvailablePlugins(ImportConfig::instance().availablePlugins());
+  setConfigs(fnCfg, id3Cfg, tagCfg, fileCfg, userActionsCfg, guiCfg, networkCfg,
+            importCfg);
+}
+
 /**
  * Set values in dialog from current configuration.
  */
@@ -387,7 +460,19 @@ void ConfigDialogPages::setConfig()
   const GuiConfig& guiCfg = GuiConfig::instance();
   const NetworkConfig& networkCfg = NetworkConfig::instance();
   const ImportConfig& importCfg = ImportConfig::instance();
+  setConfigs(fnCfg, id3Cfg, tagCfg, fileCfg, userActionsCfg, guiCfg, networkCfg,
+            importCfg);
+}
 
+/**
+ * Set values in dialog from given configurations.
+ */
+void ConfigDialogPages::setConfigs(
+    const FormatConfig& fnCfg, const FormatConfig& id3Cfg,
+    const TagConfig& tagCfg, const FileConfig& fileCfg,
+    const UserActionsConfig& userActionsCfg, const GuiConfig& guiCfg,
+    const NetworkConfig& networkCfg, const ImportConfig& importCfg)
+{
   m_fnFormatBox->fromFormatConfig(fnCfg);
   m_tagFormatBox->fromFormatConfig(id3Cfg);
   m_markTruncationsCheckBox->setChecked(tagCfg.markTruncations());
@@ -396,9 +481,30 @@ void ConfigDialogPages::setConfig()
   m_preserveTimeCheckBox->setChecked(fileCfg.preserveTime());
   m_markChangesCheckBox->setChecked(fileCfg.markChanges());
   m_coverFileNameLineEdit->setText(fileCfg.defaultCoverFileName());
+  m_nameFilterComboBox->setCurrentIndex(
+        m_nameFilterComboBox->findData(fileCfg.nameFilter()));
+  m_fileTextEncodingComboBox->setCurrentIndex(fileCfg.textEncodingIndex());
   m_onlyCustomGenresCheckBox->setChecked(tagCfg.onlyCustomGenres());
   m_genresEditModel->setStringList(tagCfg.customGenres());
-  m_quickAccessTagsModel->setBitMask(tagCfg.quickAccessFrames());
+  quint64 frameMask = tagCfg.quickAccessFrames();
+  QList<int> frameTypes = tagCfg.quickAccessFrameOrder();
+  if (frameTypes.size() != Frame::FT_LastFrame + 1) {
+    frameTypes.clear();
+    for (int i = Frame::FT_FirstFrame; i <= Frame::FT_LastFrame; ++i) {
+      frameTypes.append(i);
+    }
+  }
+  m_quickAccessTagsModel->clear();
+  foreach (int frameType, frameTypes) {
+    QStandardItem* item = new QStandardItem(
+          Frame::ExtendedType(static_cast<Frame::Type>(frameType)).getTranslatedName());
+    item->setData(frameType, Qt::UserRole);
+    item->setCheckable(true);
+    item->setCheckState((frameMask & (1ULL << frameType))
+                        ? Qt::Checked : Qt::Unchecked);
+    item->setDropEnabled(false);
+    m_quickAccessTagsModel->appendRow(item);
+  }
   m_commandsTableModel->setCommandList(userActionsCfg.contextMenuCommands());
   int idx = m_commentNameComboBox->findText(tagCfg.commentName());
   if (idx >= 0) {
@@ -488,9 +594,35 @@ void ConfigDialogPages::getConfig() const
   fileCfg.setPreserveTime(m_preserveTimeCheckBox->isChecked());
   fileCfg.setMarkChanges(m_markChangesCheckBox->isChecked());
   fileCfg.setDefaultCoverFileName(m_coverFileNameLineEdit->text());
+#if QT_VERSION >= 0x050200
+  fileCfg.setNameFilter(m_nameFilterComboBox->currentData().toString());
+#else
+  fileCfg.setNameFilter(m_nameFilterComboBox->itemData(
+                          m_nameFilterComboBox->currentIndex()).toString());
+#endif
+  fileCfg.setTextEncodingIndex(m_fileTextEncodingComboBox->currentIndex());
   tagCfg.setOnlyCustomGenres(m_onlyCustomGenresCheckBox->isChecked());
   tagCfg.setCustomGenres(m_genresEditModel->stringList());
-  tagCfg.setQuickAccessFrames(m_quickAccessTagsModel->getBitMask());
+  QList<int> frameTypes;
+  bool isStandardFrameOrder = true;
+  quint64 frameMask = 0;
+  for (int row = 0; row < m_quickAccessTagsModel->rowCount(); ++row) {
+    QModelIndex index = m_quickAccessTagsModel->index(row, 0);
+    int frameType = index.data(Qt::UserRole).toInt();
+    if (frameType != row) {
+      isStandardFrameOrder = false;
+    }
+    frameTypes.append(frameType);
+    if (m_quickAccessTagsModel->data(
+          index, Qt::CheckStateRole).toInt() == Qt::Checked) {
+      frameMask |= 1ULL << frameType;
+    }
+  }
+  if (isStandardFrameOrder) {
+    frameTypes.clear();
+  }
+  tagCfg.setQuickAccessFrames(frameMask);
+  tagCfg.setQuickAccessFrameOrder(frameTypes);
   userActionsCfg.setContextMenuCommands(m_commandsTableModel->getCommandList());
   tagCfg.setCommentName(m_commentNameComboBox->currentText());
   tagCfg.setPictureNameIndex(m_pictureNameComboBox->currentIndex());
